//+------------------------------------------------------------------+
//|                                            SlopeDirectionLine_v2 |
//|                                    Copyright (c) 2008, latimeria |
//+------------------------------------------------------------------+
#property copyright "Copyright (c) 2008, latimeria"

// You Need Custom Indicator named <Slope Direction Line>.

//parameters
extern string  ParamExp = "<<< Indicator Parameters >>>";
extern int     period=80;  // PERIOD
extern int     method=3;   // MODE_SMA
extern int     price=0;    // PRICE_CLOSE

//TradeSwitch
extern string  TradeSwitchExp = "<<< Trade Switch >>>";
extern bool    Buy = true ;
extern bool    Sell = true ;

//FakePositionSizing
extern string  FakePSExp = "<<< FakePositionSizing >>>";
extern double  FixLot=0.1; // if you don't use FakePS, this lot size is used.
extern bool    FakePositionSizing = false ;
extern double  FakePS_RiskRate = 0.8;
extern double  FakePS_ATR_Multi = 1.2;
extern int     ATR_Period=30;

//Miscellaneous Settings
extern string  MisExp = "<<< Miscellaneous Settings >>>";
extern int     Slippage = 3;
extern bool    ShowCurrentEntryPrice = true ;
extern bool    Alert_On = true ;
extern bool    Sound_On = true ;

//magic number
int MAGIC = 7777777;

//EA name
string  EANAME="SlopeDirectionLine_v2";
string  EASHORTNAME="SlopeDirLine_v2";


//+------------------------------------------------------------------+
//| start                                                            |
//+------------------------------------------------------------------+
int start()
{
	//Get Current Position
   int pos = CalculateCurrentOrders();

	//Draw Current Entry Price
   if(ShowCurrentEntryPrice)
   {
      ObjectDelete("CurrentEntry"); 
      if(pos != 0)
      {
         double CurEntryPrice = GetCurrentEntryPrice();
         DrawCurEntryPrice("CurrentEntry", Time[0], CurEntryPrice , Red);
      }
   }
   
	//Trade Only Bar Opening
   if(Volume[0]>1 || IsTradeAllowed()==false) return(0);
   
	//Culclate Indicator
   double SDL_Up = iCustom(NULL, 0, "Slope Direction Line", period, method, price, 0, 1); //Uptrend of Previous Bar
   double SDL_Dn = iCustom(NULL, 0, "Slope Direction Line", period, method, price, 1, 1); //Dntrend of Previous Bar

   //Buy
   if(pos <= 0)
   {
      if(SDL_Up < SDL_Dn)
      {
         ClosePositions();
         if(Buy) BuyOrder();
         return(0);
      }
   }

   //Sell
   if(pos >= 0)
   {
      if(SDL_Up > SDL_Dn)
      {
         ClosePositions();
         if(Sell) SellOrder();
         return(0);
      }
   }
   return(0);
}
// end of Start Function


//+------------------------------------------------------------------+
//| ClosePositions                                                   |
//+------------------------------------------------------------------+
void ClosePositions()
{
   //Total Positions
   int total = OrdersTotal();
   if (total > 0)
   {
      for (int i = total - 1; i >=0; i--)
      {
         //Select Position
         bool selected = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
         if(selected && OrderMagicNumber() == MAGIC && OrderSymbol() == Symbol())
         {
            //Get Position Type
            int type = OrderType();
            
            bool success = false;
            
            switch (type)
            {
               case OP_BUY:
                  double bid = MarketInfo(OrderSymbol(), MODE_BID);
                  success = OrderClose(OrderTicket(),OrderLots(), bid, 3, CLR_NONE);
                  if(Alert_On) Alert(EASHORTNAME,"_ClosePosition ", Symbol(), " at ", Bid," Lots ",OrderLots()); 
                  if(Sound_On) PlaySound("alert2.wav");
                  break;
               case OP_SELL:
                  double ask = MarketInfo(OrderSymbol(), MODE_ASK);
                  success = OrderClose(OrderTicket(),OrderLots(), ask, 3, CLR_NONE);
                  if(Alert_On) Alert(EASHORTNAME,"_ClosePosition ", Symbol(), " at ", Ask," Lots ",OrderLots());      
                  if(Sound_On) PlaySound("alert2.wav");
                  break;
               case OP_BUYSTOP:
               case OP_BUYLIMIT:
               case OP_SELLSTOP:
               case OP_SELLLIMIT:
                  success = OrderDelete(OrderTicket());
                  break;
            }
            
            if (!success)
            {
               Print("Error code = " + GetLastError());
            }
         }
      }
   }
} // end of ClosePositions Function

//+------------------------------------------------------------------+
//| Culculate Current Open Position                                  |
//+------------------------------------------------------------------+
int CalculateCurrentOrders()
{
   //Open Positionsi+:Buy -:Sellj
   int i;
   int pos=0;

   // Buy Position
   int pos_buy=0;
   for(i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES) == false) break;
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MAGIC)
      {
         if(OrderType() == OP_BUY) pos_buy++ ;
      }
   }

   //Sell Position
   int pos_sell=0;
   for(i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES) == false) break;
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MAGIC)
      {
         if(OrderType() == OP_SELL) pos_sell--;
      }
   }

   //Culculate pos
   if(pos_buy>0) pos = pos_buy;
   if(pos_sell<0) pos = pos_sell;
   if(pos_sell == 0 && pos_buy == 0) pos = 0;

   return(pos);
} // end of CalculateCurrentOrders

//+------------------------------------------------------------------+
//| Buy Order                                                        |
//+------------------------------------------------------------------+
void BuyOrder()
{   
   double lot = FakePositionSizing();

   if(lot<0.1) return(0); // if you use microlot(0.01), change minimum lot to 0.01
   OrderSend(Symbol(), OP_BUY, lot, Ask, Slippage, 0, 0, EASHORTNAME+"_"+Symbol()+"_B", MAGIC, 0, Green);
   if(Alert_On) Alert(EASHORTNAME, "_Buy ", Symbol(), " at ", Ask," Lots ",lot);
   if(Sound_On) PlaySound("news.wav");      
}

//+------------------------------------------------------------------+
//| Sell Order                                                       |
//+------------------------------------------------------------------+
void SellOrder()
{
   double lot = FakePositionSizing();

   if(lot<0.1)return(0); // if you use microlot(0.01), change minimum lot to 0.01
   OrderSend(Symbol(), OP_SELL, lot, Bid, Slippage, 0, 0, EASHORTNAME+"_"+Symbol()+"_S", MAGIC, 0, Red);
   if(Alert_On) Alert(EASHORTNAME, "_Sell ", Symbol(), " at ", Bid," Lots ",lot);
   if(Sound_On) PlaySound("news.wav");      
}

//+------------------------------------------------------------------+
//| Fake_Position_Sizing                                             |
//+------------------------------------------------------------------+
double FakePositionSizing()
{
   double lot = FixLot;
   
   //FakePositionSizing
   double StopLoss = iATR(NULL,0,ATR_Period,1) * FakePS_ATR_Multi ;

   if(FakePositionSizing)
   {
      double RiskAmount = AccountFreeMargin() * (FakePS_RiskRate/100) ;
      int SLpips = StopLoss/Point ;
      double PT = profit_unit(); // call function
      lot = RiskAmount/(PT * SLpips) ;
   }

   lot = NormalizeDouble(lot,1) ; // if you use microlot(0.01), change digit to 2

   return(lot);
}

//+------------------------------------------------------------------+
//| profit_unit                                                      |
//+------------------------------------------------------------------+
double profit_unit()
{
    double lot_size    =MarketInfo(Symbol(),MODE_LOTSIZE); //
    string base_currecy=StringSubstr(Symbol(),0,3);
    int    symbol_len  =StringLen(Symbol());
    string term_currecy="USD";
    if(symbol_len==6)
        term_currecy=StringSubstr(Symbol(),3,3);
    string term_pair =term_currecy+AccountCurrency();
    string xterm_pair=AccountCurrency()+term_currecy;
    string base_pair =base_currecy+AccountCurrency();
    double term_unit =1.0;
    if(base_currecy==AccountCurrency())
    { 
        term_unit=1.0/Bid;
    }
    else if(term_currecy!=AccountCurrency())
    {
        term_unit=iClose(term_pair,0,0);
        if(term_unit<=0)
        {
            term_unit=iClose(xterm_pair,0,0);
            if(term_unit<=0)
            {
                term_unit=iClose(base_pair,0,0);
            }
            else
            {
               term_unit=1.0/term_unit;
            }
        }
    }
    return(lot_size*Point*term_unit); // returns Profit/Loss per 1.0 lot * 1pip by your accout currency.
}

//+------------------------------------------------------------------+
//| GetCurrentEntryPrice                                             |
//+------------------------------------------------------------------+
double GetCurrentEntryPrice()
{
   int i; double CurEntryPrice;

   for(i=0; i<OrdersTotal(); i++)
   {
      bool selected = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(selected && OrderMagicNumber() == MAGIC && OrderSymbol() == Symbol())
      {
         CurEntryPrice = OrderOpenPrice();
      }
   }
return(CurEntryPrice);
}

//+------------------------------------------------------------------+
//| GetCurrentEntryType                                              |
//+------------------------------------------------------------------+
int GetCurrentEntryType()
{
   int i; int CurEntryType;

   for(i=0; i<OrdersTotal(); i++)
   {
      bool selected = OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(selected && OrderMagicNumber() == MAGIC && OrderSymbol() == Symbol())
      {
         if(OrderType() == OP_BUY) CurEntryType = 1;
         if(OrderType() == OP_SELL) CurEntryType = 2;
      }
   }
return(CurEntryType);
}

//+------------------------------------------------------------------+
//| DrawCurEntryPrice                                                |
//+------------------------------------------------------------------+
void DrawCurEntryPrice(string name, datetime TIME, double PRICE, color COLOR)
  {
   if(GetCurrentEntryType() == 1) COLOR = Lime;
   if(GetCurrentEntryType() == 2) COLOR = Red;

   if(ObjectFind(name) == -1)
     {
       ObjectCreate(name, OBJ_ARROW, 0, TIME, PRICE);
       ObjectSet(name, OBJPROP_COLOR, COLOR);
       ObjectSet(name, OBJPROP_WIDTH, 1);
       ObjectSet(name, OBJPROP_ARROWCODE, SYMBOL_RIGHTPRICE);
     }
   else
     {
       ObjectSet(name, OBJPROP_TIME1, TIME);
       ObjectSet(name, OBJPROP_PRICE1, PRICE);
       ObjectSet(name, OBJPROP_COLOR, COLOR);
       ObjectSet(name, OBJPROP_WIDTH, 1);
       ObjectSet(name, OBJPROP_ARROWCODE, SYMBOL_RIGHTPRICE);
     } 
  }

